@@ -62,6 +62,7 @@ from .GXDLMSConfirmedServiceError import GXDLMSConfirmedServiceError
 from .MBusEncryptionMode import MBusEncryptionMode
 from .MBusCommand import MBusCommand
 from .enums.Standard import Standard
+from .GXDLMSExceptionResponse import GXDLMSExceptionResponse
 
 # pylint: disable=too-many-public-methods,too-many-function-args
 class GXDLMS:
@@ -482,7 +483,7 @@ class GXDLMS:
         key = None
         cipher = p.settings.cipher
         if (p.settings.negotiatedConformance & Conformance.GENERAL_PROTECTION) == Conformance.NONE:
-            if cipher.dedicatedKey and (not p.settings.isServer or (p.settings.connected & ConnectionState.DLMS) != 0):
+            if cipher.dedicatedKey and (p.settings.connected & ConnectionState.DLMS) != 0:
                 cmd = cls.getDedMessage(p.command)
                 key = cipher.dedicatedKey
             else:
@@ -1085,7 +1086,7 @@ class GXDLMS:
             settings.setBlockIndex(number)
         expectedIndex = settings.blockIndex
         if number != expectedIndex:
-            raise Exception("Invalid Block number. It is " + number + " and it should be " + expectedIndex + ".")
+            raise Exception("Invalid Block number. It is " + str(number) + " and it should be " + str(expectedIndex) + ".")
         if (reply.moreData & RequestTypes.FRAME) != 0:
             cls.getDataFromBlock(data, index)
             return False
@@ -1174,7 +1175,7 @@ class GXDLMS:
             elif type_ == SingleReadResponse.BLOCK_NUMBER:
                 number = data.getUInt16()
                 if number != settings.blockIndex:
-                    raise Exception("Invalid Block number. It is " + number + " and it should be " + settings.blockIndex + ".")
+                    raise Exception("Invalid Block number. It is " + str(number) + " and it should be " + str(settings.blockIndex) + ".")
                 settings.increaseBlockIndex()
                 reply.moreData = (RequestTypes(reply.moreData | RequestTypes.DATABLOCK))
             else:
@@ -1498,7 +1499,7 @@ class GXDLMS:
                     settings.setBlockIndex(0)
                 expectedIndex = settings.blockIndex
                 if number != expectedIndex:
-                    raise ValueError("Invalid Block number. It is " + number + " and it should be " + expectedIndex + ".")
+                    raise ValueError("Invalid Block number. It is " + str(number) + " and it should be " + str(expectedIndex) + ".")
             ch = reply.data.getUInt8()
             if ch != 0:
                 reply.error = reply.data.getUInt8()
@@ -1709,10 +1710,6 @@ class GXDLMS:
             cmd in (Command.READ_RESPONSE, Command.GET_RESPONSE, Command.METHOD_RESPONSE, Command.DATA_NOTIFICATION) and (data.moreData == RequestTypes.NONE or data.peek):
             cls.getValueFromData(settings, data)
 
-    @classmethod
-    def handleExceptionResponse(cls, data):
-        raise Exception(StateError(data.data.getUInt8() - 1), ExceptionServiceError(data.data.getUInt8() - 1))
-
     @classmethod
     def handleConfirmedServiceError(cls, data):
         if data.xml:
@@ -1737,6 +1734,25 @@ class GXDLMS:
             type_ = data.data.getUInt8()
             raise GXDLMSConfirmedServiceError(service, type_, data.data.getUInt8())
 
+    @classmethod
+    def handleExceptionResponse(cls, data):
+        state = data.data.getUInt8()
+        error = ExceptionServiceError(data.data.getUInt8())
+        value = None
+        if error == ExceptionServiceError.INVOCATION_COUNTER_ERROR and data.data.available() > 3:
+            value = data.data.getUInt32()
+        if data.xml:
+            data.xml.appendStartTag(Command.EXCEPTION_RESPONSE)
+            if data.xml.outputType == TranslatorOutputType.STANDARD_XML:
+                data.xml.appendLine(TranslatorTags.STATE_ERROR, None, TranslatorStandardTags.stateErrorToString(state))
+                data.xml.appendLine(TranslatorTags.SERVICE_ERROR, None, TranslatorStandardTags.exceptionServiceErrorToString(error))
+            else:
+                data.xml.appendLine(TranslatorTags.STATE_ERROR, None, TranslatorSimpleTags.stateErrorToString(state))
+                data.xml.appendLine(TranslatorTags.SERVICE_ERROR, None, TranslatorSimpleTags.exceptionServiceErrorToString(error))
+            data.xml.appendEndTag(Command.EXCEPTION_RESPONSE)
+        else:
+            raise GXDLMSExceptionResponse(state, error, value)
+
     @classmethod
     def handleGloDedRequest(cls, settings, data):
         if settings.cipher is None:
@@ -1768,7 +1784,7 @@ class GXDLMS:
             bb = GXByteBuffer(data.data)
             data.data.size = data.data.position = index
             p = None
-            if settings.cipher.dedicatedKey and (not settings.isServer or settings.connected == ConnectionState.DLMS):
+            if settings.cipher.dedicatedKey and (settings.connected & ConnectionState.DLMS) != 0:
                 p = AesGcmParameter(0, settings.sourceSystemTitle, settings.cipher.dedicatedKey, settings.cipher.authenticationKey)
             else:
                 p = AesGcmParameter(0, settings.sourceSystemTitle, settings.cipher.blockCipherKey, settings.cipher.authenticationKey)
@@ -1983,10 +1999,10 @@ class GXDLMS:
     @classmethod
     def parseSnrmUaResponse(cls, data, limits):
         if data.available() == 0:
-            limits.setMaxInfoTX(GXDLMSLimits.DEFAULT_MAX_INFO_TX)
-            limits.setMaxInfoRX(GXDLMSLimits.DEFAULT_MAX_INFO_RX)
-            limits.setWindowSizeTX(GXDLMSLimits.DEFAULT_WINDOWS_SIZE_TX)
-            limits.setWindowSizeRX(GXDLMSLimits.DEFAULT_WINDOWS_SIZE_RX)
+            limits.maxInfoTX = GXDLMSLimits.DEFAULT_MAX_INFO_TX
+            limits.maxInfoRX = GXDLMSLimits.DEFAULT_MAX_INFO_RX
+            limits.windowSizeTX = GXDLMSLimits.DEFAULT_WINDOWS_SIZE_TX
+            limits.windowSizeRX = GXDLMSLimits.DEFAULT_WINDOWS_SIZE_RX
         else:
             data.getUInt8()
             data.getUInt8()
