@@ -31,15 +31,9 @@
 #  This code is licensed under the GNU General Public License v2.
 #  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
 # ---------------------------------------------------------------------------
-import sys
-#pylint: disable=no-name-in-module
-if sys.version_info < (3, 0):
-    __base = object
-else:
-    from enum import IntEnum
-    __base = IntEnum
+from ..GXIntEnum import GXIntEnum
 
-class InterfaceType(__base):
+class InterfaceType(GXIntEnum):
     """InterfaceType enumerates the usable types of connection in GuruxDLMS."""
     #pylint: disable=too-few-public-methods
 
