@@ -143,9 +143,9 @@ class GXDLMSPppSetup(GXDLMSObject, IGXDLMSBase):
                 for it in self.lcpOptions:
                     data.setUInt8(DataType.STRUCTURE)
                     data.setUInt8(3)
-                    _GXCommon.setData(data, DataType.UINT8, it.type_.value)
-                    _GXCommon.setData(data, DataType.UINT8, it.length)
-                    _GXCommon.setData(data, _GXCommon.getDLMSDataType(it.data), it.data)
+                    _GXCommon.setData(settings, data, DataType.UINT8, it.type_.value)
+                    _GXCommon.setData(settings, data, DataType.UINT8, it.length)
+                    _GXCommon.setData(settings, data, _GXCommon.getDLMSDataType(it.data), it.data)
             ret = data.array()
         elif e.index == 4:
             data = GXByteBuffer()
@@ -157,17 +157,17 @@ class GXDLMSPppSetup(GXDLMSObject, IGXDLMSBase):
                 for it in self.ipcpOptions:
                     data.setUInt8(DataType.STRUCTURE)
                     data.setUInt8(3)
-                    _GXCommon.setData(data, DataType.UINT8, it.type_.value)
-                    _GXCommon.setData(data, DataType.UINT8, it.length)
-                    _GXCommon.setData(data, _GXCommon.getDLMSDataType(it.data), it.data)
+                    _GXCommon.setData(settings, data, DataType.UINT8, it.type_.value)
+                    _GXCommon.setData(settings, data, DataType.UINT8, it.length)
+                    _GXCommon.setData(settings, data, _GXCommon.getDLMSDataType(it.data), it.data)
             ret = data.array()
         elif e.index == 5:
             if self.userName:
                 data = GXByteBuffer()
                 data.setUInt8(DataType.STRUCTURE)
                 data.setUInt8(2)
-                _GXCommon.setData(data, DataType.OCTET_STRING, self.userName)
-                _GXCommon.setData(data, DataType.OCTET_STRING, self.password)
+                _GXCommon.setData(settings, data, DataType.OCTET_STRING, self.userName)
+                _GXCommon.setData(settings, data, DataType.OCTET_STRING, self.password)
                 ret = data.array()
             else:
                 ret = None
@@ -237,23 +237,23 @@ class GXDLMSPppSetup(GXDLMSObject, IGXDLMSBase):
 
     def save(self, writer):
         writer.writeElementString("PHYReference", self.phyReference)
+        writer.writeStartElement("LCPOptions")
         if self.lcpOptions:
-            writer.writeStartElement("LCPOptions")
             for it in self.lcpOptions:
                 writer.writeStartElement("Item")
                 writer.writeElementString("Type", it.type_)
                 writer.writeElementString("Length", it.length)
                 writer.writeElementObject("Data", it.data)
                 writer.writeEndElement()
-            writer.writeEndElement()
+        writer.writeEndElement()
+        writer.writeStartElement("IPCPOptions")
         if self.ipcpOptions:
-            writer.writeStartElement("IPCPOptions")
             for it in self.ipcpOptions:
                 writer.writeStartElement("Item")
                 writer.writeElementString("Type", it.type_)
                 writer.writeElementString("Length", it.length)
                 writer.writeElementObject("Data", it.data)
                 writer.writeEndElement()
-            writer.writeEndElement()
+        writer.writeEndElement()
         writer.writeElementString("UserName", GXByteBuffer.hex(self.userName))
         writer.writeElementString("Password", GXByteBuffer.hex(self.password))
